@@ -26,11 +26,16 @@ import (
 	"launchpad.net/ubuntu-push/server/store"
 )
 
+type SessionTracker interface {
+	// SessionId
+	SessionId() string
+}
+
 // Broker is responsible for registring sessions and delivering messages
 // through them.
 type Broker interface {
 	// Register the session.
-	Register(connMsg *protocol.ConnectMsg, sessionId string) (BrokerSession, error)
+	Register(connMsg *protocol.ConnectMsg, track SessionTracker) (BrokerSession, error)
 	// Unregister the session.
 	Unregister(BrokerSession)
 }
@@ -55,7 +60,7 @@ var ErrNop = errors.New("nothing to send")
 // LevelsMap is the type for holding channel levels for session.
 type LevelsMap map[store.InternalChannelId]int64
 
-// GetInfoString helps retrivieng a string out of a protocol.ConnectMsg.Info
+// GetInfoString helps retrieveng a string out of a protocol.ConnectMsg.Info.
 func GetInfoString(msg *protocol.ConnectMsg, name, defaultVal string) (string, error) {
 	v, ok := msg.Info[name]
 	if !ok {
@@ -68,6 +73,19 @@ func GetInfoString(msg *protocol.ConnectMsg, name, defaultVal string) (string, e
 	return s, nil
 }
 
+// GetInfoInt helps retrieving an integer out of a protocol.ConnectMsg.Info.
+func GetInfoInt(msg *protocol.ConnectMsg, name string, defaultVal int) (int, error) {
+	v, ok := msg.Info[name]
+	if !ok {
+		return defaultVal, nil
+	}
+	n, ok := v.(float64)
+	if !ok {
+		return -1, ErrUnexpectedValue
+	}
+	return int(n), nil
+}
+
 // BrokerSession holds broker session state.
 type BrokerSession interface {
 	// SessionChannel returns the session control channel
