@@ -46,6 +46,8 @@ type FullBroker interface {
 type CommonBrokerSuite struct {
 	// Build the broker for testing.
 	MakeBroker func(store.PendingStore, broker.BrokerConfig, logger.Logger) FullBroker
+	// Build a session tracker for testing.
+	MakeTracker func(sessionId string) broker.SessionTracker
 	// Let us get to a session under the broker.
 	RevealSession func(broker.Broker, string) broker.BrokerSession
 	// Let us get to a broker.BroadcastExchange from an Exchange.
@@ -64,12 +66,12 @@ var testBrokerConfig = &testing.TestBrokerConfig{10, 5}
 
 func (s *CommonBrokerSuite) TestSanity(c *C) {
 	sto := store.NewInMemoryPendingStore()
-	b := s.MakeBroker(sto, testBrokerConfig, nil)
+	b := s.MakeBroker(sto, testBrokerConfig, s.testlog)
 	c.Check(s.RevealSession(b, "FOO"), IsNil)
 }
 
 func (s *CommonBrokerSuite) TestStartStop(c *C) {
-	b := s.MakeBroker(nil, testBrokerConfig, nil)
+	b := s.MakeBroker(nil, testBrokerConfig, s.testlog)
 	b.Start()
 	c.Check(b.Running(), Equals, true)
 	b.Start()
@@ -80,7 +82,7 @@ func (s *CommonBrokerSuite) TestStartStop(c *C) {
 
 func (s *CommonBrokerSuite) TestRegistration(c *C) {
 	sto := store.NewInMemoryPendingStore()
-	b := s.MakeBroker(sto, testBrokerConfig, nil)
+	b := s.MakeBroker(sto, testBrokerConfig, s.testlog)
 	b.Start()
 	defer b.Stop()
 	sess, err := b.Register(&protocol.ConnectMsg{
@@ -91,7 +93,7 @@ func (s *CommonBrokerSuite) TestRegistration(c *C) {
 			"device":  "model",
 			"channel": "daily",
 		},
-	}, "s1")
+	}, s.MakeTracker("s1"))
 	c.Assert(err, IsNil)
 	c.Assert(s.RevealSession(b, "dev-1"), Equals, sess)
 	c.Assert(sess.DeviceIdentifier(), Equals, "dev-1")
@@ -103,33 +105,33 @@ func (s *CommonBrokerSuite) TestRegistration(c *C) {
 	}))
 	b.Unregister(sess)
 	// just to make sure the unregister was processed
-	_, err = b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: ""}, "s2")
+	_, err = b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: ""}, s.MakeTracker("s2"))
 	c.Assert(err, IsNil)
 	c.Check(s.RevealSession(b, "dev-1"), IsNil)
 }
 
 func (s *CommonBrokerSuite) TestRegistrationBrokenLevels(c *C) {
 	sto := store.NewInMemoryPendingStore()
-	b := s.MakeBroker(sto, testBrokerConfig, nil)
+	b := s.MakeBroker(sto, testBrokerConfig, s.testlog)
 	b.Start()
 	defer b.Stop()
-	_, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1", Levels: map[string]int64{"z": 5}}, "s1")
+	_, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1", Levels: map[string]int64{"z": 5}}, s.MakeTracker("s1"))
 	c.Check(err, FitsTypeOf, &broker.ErrAbort{})
 }
 
 func (s *CommonBrokerSuite) TestRegistrationInfoErrors(c *C) {
 	sto := store.NewInMemoryPendingStore()
-	b := s.MakeBroker(sto, testBrokerConfig, nil)
+	b := s.MakeBroker(sto, testBrokerConfig, s.testlog)
 	b.Start()
 	defer b.Stop()
 	info := map[string]interface{}{
 		"device": -1,
 	}
-	_, err := b.Register(&protocol.ConnectMsg{Type: "connect", Info: info}, "s1")
+	_, err := b.Register(&protocol.ConnectMsg{Type: "connect", Info: info}, s.MakeTracker("s1"))
 	c.Check(err, Equals, broker.ErrUnexpectedValue)
 	info["device"] = "m"
 	info["channel"] = -1
-	_, err = b.Register(&protocol.ConnectMsg{Type: "connect", Info: info}, "s2")
+	_, err = b.Register(&protocol.ConnectMsg{Type: "connect", Info: info}, s.MakeTracker("s2"))
 	c.Check(err, Equals, broker.ErrUnexpectedValue)
 }
 
@@ -138,10 +140,10 @@ func (s *CommonBrokerSuite) TestRegistrationFeedPending(c *C) {
 	notification1 := json.RawMessage(`{"m": "M"}`)
 	muchLater := time.Now().Add(10 * time.Minute)
 	sto.AppendToChannel(store.SystemInternalChannelId, notification1, muchLater)
-	b := s.MakeBroker(sto, testBrokerConfig, nil)
+	b := s.MakeBroker(sto, testBrokerConfig, s.testlog)
 	b.Start()
 	defer b.Stop()
-	sess, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, "s1")
+	sess, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, s.MakeTracker("s1"))
 	c.Assert(err, IsNil)
 	c.Check(len(sess.SessionChannel()), Equals, 2)
 }
@@ -151,7 +153,7 @@ func (s *CommonBrokerSuite) TestRegistrationFeedPendingError(c *C) {
 	b := s.MakeBroker(sto, testBrokerConfig, s.testlog)
 	b.Start()
 	defer b.Stop()
-	_, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, "s1")
+	_, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, s.MakeTracker("s1"))
 	c.Assert(err, IsNil)
 	// but
 	c.Check(s.testlog.Captured(), Matches, "ERROR unsuccessful, get channel snapshot for 0 \\(cachedOk=true\\): get channel snapshot fail\n")
@@ -164,13 +166,13 @@ func clearOfPending(c *C, sess broker.BrokerSession) {
 
 func (s *CommonBrokerSuite) TestRegistrationLastWins(c *C) {
 	sto := store.NewInMemoryPendingStore()
-	b := s.MakeBroker(sto, testBrokerConfig, nil)
+	b := s.MakeBroker(sto, testBrokerConfig, s.testlog)
 	b.Start()
 	defer b.Stop()
-	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, "s1")
+	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, s.MakeTracker("s1"))
 	c.Assert(err, IsNil)
 	clearOfPending(c, sess1)
-	sess2, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, "s2")
+	sess2, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, s.MakeTracker("s2"))
 	c.Assert(err, IsNil)
 	// previous session got signaled by sending nil on its channel
 	var sentinel broker.Exchange
@@ -186,7 +188,7 @@ func (s *CommonBrokerSuite) TestRegistrationLastWins(c *C) {
 	c.Assert(s.RevealSession(b, "dev-1"), Equals, sess2)
 	b.Unregister(sess1)
 	// just to make sure the unregister was processed
-	_, err = b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: ""}, "s3")
+	_, err = b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: ""}, s.MakeTracker("s3"))
 	c.Assert(err, IsNil)
 	c.Check(s.RevealSession(b, "dev-1"), Equals, sess2)
 }
@@ -195,13 +197,13 @@ func (s *CommonBrokerSuite) TestBroadcast(c *C) {
 	sto := store.NewInMemoryPendingStore()
 	notification1 := json.RawMessage(`{"m": "M"}`)
 	decoded1 := map[string]interface{}{"m": "M"}
-	b := s.MakeBroker(sto, testBrokerConfig, nil)
+	b := s.MakeBroker(sto, testBrokerConfig, s.testlog)
 	b.Start()
 	defer b.Stop()
-	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, "s1")
+	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, s.MakeTracker("s1"))
 	c.Assert(err, IsNil)
 	clearOfPending(c, sess1)
-	sess2, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-2"}, "s2")
+	sess2, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-2"}, s.MakeTracker("s2"))
 	c.Assert(err, IsNil)
 	clearOfPending(c, sess2)
 	// add notification to channel *after* the registrations
@@ -258,7 +260,7 @@ func (s *CommonBrokerSuite) TestBroadcastFail(c *C) {
 	b := s.MakeBroker(sto, testBrokerConfig, s.testlog)
 	b.Start()
 	defer b.Stop()
-	sess, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, "s1")
+	sess, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, s.MakeTracker("s1"))
 	c.Assert(err, IsNil)
 	clearOfPending(c, sess)
 	b.Broadcast(store.SystemInternalChannelId)
@@ -276,13 +278,13 @@ func (s *CommonBrokerSuite) TestUnicast(c *C) {
 	notification2 := json.RawMessage(`{"m": "M2"}`)
 	chanId1 := store.UnicastInternalChannelId("dev1", "dev1")
 	chanId2 := store.UnicastInternalChannelId("dev2", "dev2")
-	b := s.MakeBroker(sto, testBrokerConfig, nil)
+	b := s.MakeBroker(sto, testBrokerConfig, s.testlog)
 	b.Start()
 	defer b.Stop()
-	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev1"}, "s1")
+	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev1"}, s.MakeTracker("s1"))
 	c.Assert(err, IsNil)
 	clearOfPending(c, sess1)
-	sess2, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev2"}, "s2")
+	sess2, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev2"}, s.MakeTracker("s2"))
 	c.Assert(err, IsNil)
 	clearOfPending(c, sess2)
 	// add notification to channel *after* the registrations
@@ -310,10 +312,10 @@ func (s *CommonBrokerSuite) TestGetAndDrop(c *C) {
 	sto := store.NewInMemoryPendingStore()
 	notification1 := json.RawMessage(`{"m": "M1"}`)
 	chanId1 := store.UnicastInternalChannelId("dev3", "dev3")
-	b := s.MakeBroker(sto, testBrokerConfig, nil)
+	b := s.MakeBroker(sto, testBrokerConfig, s.testlog)
 	b.Start()
 	defer b.Stop()
-	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev3"}, "s1")
+	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev3"}, s.MakeTracker("s1"))
 	c.Assert(err, IsNil)
 	muchLater := store.Metadata{Expiration: time.Now().Add(10 * time.Minute)}
 	sto.AppendToUnicastChannel(chanId1, "app1", notification1, "msg1", muchLater)
@@ -338,7 +340,7 @@ func (s *CommonBrokerSuite) TestGetAndDropErrors(c *C) {
 	b := s.MakeBroker(sto, testBrokerConfig, s.testlog)
 	b.Start()
 	defer b.Stop()
-	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev3"}, "s1")
+	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev3"}, s.MakeTracker("s1"))
 	c.Assert(err, IsNil)
 	_, _, err = sess1.Get(chanId1, false)
 	c.Assert(err, ErrorMatches, "get channel snapshot fail")
@@ -352,10 +354,10 @@ func (s *CommonBrokerSuite) TestGetAndDropErrors(c *C) {
 
 func (s *CommonBrokerSuite) TestSessionFeed(c *C) {
 	sto := store.NewInMemoryPendingStore()
-	b := s.MakeBroker(sto, testBrokerConfig, nil)
+	b := s.MakeBroker(sto, testBrokerConfig, s.testlog)
 	b.Start()
 	defer b.Stop()
-	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev3"}, "s1")
+	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev3"}, s.MakeTracker("s1"))
 	c.Assert(err, IsNil)
 	clearOfPending(c, sess1)
 	bcast := &broker.BroadcastExchange{ChanId: store.SystemInternalChannelId, TopLevel: 99}
